
sorted-intersect
================


Installation
------------
Download from pypi:

```
pip install sortedintersect
```
    

To build from source:

```
git clone https://github.com/kcleal/sortedintersect
cd sortedintersect
pip install .
```

Overview
--------

Common usage is to check if a point or interval overlaps a reference set:

```python
from sortedintersect import IntervalSet

# intervals without data
itv = IntervalSet()
itv.add(0, 10)
itv.search_point(1)
# >>> [(0, 10)]

# Return a boolean only (faster)
itv = IntervalSet(bool_only=True)
itv.add(0, 10)
itv.search_point(1)
# >>> True

# search intervals with additional data
itv = IntervalSet(with_data=True)
itv.add(0, 10, 'interval1')
itv.add(20, 30, {'a': 1})
itv.search_interval(1, 2)
# >>> [(0, 10, 'interval1')]
itv.search_point(20)
# >>> [(20, 30, {'a': 1})]


itv = IntervalSet()
itv.add(-1, 1)
itv.add(2, 4)
itv.add(9, 10)
assert tuple(itv.search_point(10)) == ((9, 10),)
assert tuple(itv.search_point(0)) == ((-1, 1),)

# set distance threshold
itv = IntervalSet(distance_threshold=500_000)
```


An interval tree is often used for this purpose relying on binary search in log(N) time.
However, if the reference intervals are sorted ahead of time, and query intervals 
are roughly sorted, then a simple plane-sweep algorithm can be used, relying on a linear search instead.
This situation arises when processing a sorted alignment or vcf file and checking against a sorted reference
interval set, for example. Sorted-intersect uses a simple heuristic (a distance measure) to decide when to use
binary-search adn when to use a plane-sweep.
When a query-point is tested, its position is recorded. When a subsequent query point is tested, if the distance to
the previous point is below a threshold, then a plane sweep is used on the reference intervals instead of a binary search.



Benchmarks
----------

sortedintersect was compared to popular python implementations based on interval trees, see tests/benchmark.py for details.

random1 - randomly generated intervals using a python script
random2 - 1million random intervals of 1kb in size over chromosome 1, generated by bedtools
reads+genes - reads from a 2mb region converted to bed format, then intersected with ucsc genes from hg19

![alt text](https://github.com/kcleal/sortedintersect/blob/master/tests/benchmark.png)

| library         |   time (s) |   intersections | test        | shuffle   |
|:----------------|-----------:|----------------:|:------------|:----------|
| sortedintersect |  0.0116289 |            2947 | random1     | True      |
| quicksect       |  0.0148401 |            2947 | random1     | True      |
| cgranges        |  0.0177381 |            2878 | random1     | True      |
| ncls            |  0.026356  |            2947 | random1     | True      |
| sortedintersect |  0.713765  |         8001594 | random2     | True      |
| quicksect       |  1.28989   |         8001594 | random2     | True      |
| cgranges        |  1.01043   |         7993517 | random2     | True      |
| ncls            |  1.36818   |         8001594 | random2     | True      |
| sortedintersect |  0.112607  |          320618 | reads+genes | True      |
| quicksect       |  0.127229  |          320618 | reads+genes | True      |
| cgranges        |  0.160962  |          320610 | reads+genes | True      |
| ncls            |  0.237285  |          320618 | reads+genes | True      |
| sortedintersect |  0.0111213 |            2916 | random1     | False     |
| quicksect       |  0.0144    |            2916 | random1     | False     |
| cgranges        |  0.0177028 |            2847 | random1     | False     |
| ncls            |  0.0270391 |            2916 | random1     | False     |
| sortedintersect |  0.59601   |         8001594 | random2     | False     |
| quicksect       |  0.658922  |         8001594 | random2     | False     |
| cgranges        |  0.879546  |         7993517 | random2     | False     |
| ncls            |  1.15896   |         8001594 | random2     | False     |
| sortedintersect |  0.100809  |          320618 | reads+genes | False     |
| quicksect       |  0.125237  |          320618 | reads+genes | False     |
| cgranges        |  0.153327  |          320610 | reads+genes | False     |
| ncls            |  0.230667  |          320618 | reads+genes | False     |


Limitations
-----------

- Reference queries must be added in sorted order
- Reference intervals can not be deleted
